;+
;Language : IDL 8.4
;================================================================================
;The AAS gives permission to anyone who wishes to use these subroutines to run their own calculations.
;Permission to republish or reuse these routines should be directed to permissions@aas.org.
;
;Note that the AAS does not take responsibility for the content of the source code. Potential users should
;be wary of applying the code to conditions that the code was not written to model and the accuracy of the
;code may be affected when compiled and executed on different systems.
;================================================================================
; NAME:
;       BANYAN_MASS
;
; INSTALLATION INSTRUCTIONS :
;       Set a variable names PIDL_IDL_SAV in your bash profile that points towards a directory containing
;         the "baraffe_interpol_input.sav" file. An example with a bash profile would be :  
;         export PIDL_IDL_SAV="~/IDL_sav/"
;         
;       WINDOWS USERS : Instead, open the "path_library.pro" routine, add your (***lower case***) computer name
;         in the case statement that corresponds to "idl_sav", and have it return that directory.
;       
; PURPOSE:
;       This routine uses the J, H, Ks, W1 and W2 apparent magnitudes and the distance of a given object, along with
;       an estimated age range to determine an 1-sigma mass range using BT-Settl isochrones (Allard et al. 2013,
;       Baraffe et al. 2003) in a likelihood analysis. Errors on the distance and photometry, when input, are propagated
;       to the estimated mass range.
;
; CALLING SEQUENCE:
;       BANYAN_MASS, age_0, age_1, dist, edist, mass_0, mass_1[, J=J, H=H, K=K, W1=W1, W2=W2, EJ=EJ, EH=EH,
;         EK=EK, EW1=EW1, EW2=EW2, MAX_MASS=max_mass, BINARY=binary, S2M=s2m, SWISE=swise
; 
; EXAMPLE :
; 
;       Compute the mass of SDSS J1110+0116 (110-130 Myr) :
;       
;       IDL> banyan_mass,110.,130.,J=16.16,eJ=0.01,H=16.2,eH=0.02,K=16.05,eK=0.03,W1=15.44,eW1=0.04,W2=13.92,eW2=0.04,dist=19.19,edist=0.44,out_struct=out_struct,pdf_mass=pdf_mass
;       IDL> plot,xtitle='M_Jup',ytitle='Probability Density Function',pdf_mass[*,0]*1047.6,pdf_mass[*,1],xrange=[10.5,11.5] & oplot,pdf_mass[*,0]*1047.6,pdf_mass[*,1],/ps,color=255
;       IDL> print,'Mass is $'+string(out_struct.mass,format='(F5.2)')+'_{-'+string(out_struct.emass[0],format='(F4.2)')+'}^{+'+string(out_struct.emass[1],format='(F4.2)')+'}'+'$ M_Jup'
;         Mass is $10.92_{-0.06}^{+0.04}$ M_Jup
;       
; INPUTS:
;       AGE_0  - The lower value of the estimated age range.
;       AGE_1  - The higher value of the estimated age range.
;       DIST   - The distance measurement.
;       EDIST  - The error on the distance measurement.
;
; OPTIONAL INPUT KEYWORD:
;       J - Apparent 2MASS J magnitude.
;       H - Apparent 2MASS H magnitude.
;       K - Apparent 2MASS Ks magnitude.
;       W1 - Apparent WISE W1 magnitude.
;       W2 - Apparent WISE W2 magnitude.
;       EJ - Error on apparent 2MASS J magnitude.
;       EH - Error on apparent 2MASS H magnitude.
;       EK - Error on apparent 2MASS Ks magnitude.
;       EW1 - Error on apparent WISE W1 magnitude.
;       EW2 - Error on apparent WISE W2 magnitude.
;       S2M - 2MASS structure containing photometry.
;       SWISE - WISE structure containing photometry.
;       MAX_MASS - The maximum mass that is tested on the grid, in Solar masses. By default this is set to 0.2 Solar masses.
;                  If a star with a larger mass is tested with this program, results will not be good.
;       /BINARY - Set this keyword to remove 0.7 magnitudes to every channel if the tested object is an equal-luminosity binary.
;
; OUTPUTS:
;       M - The most probable mass (Solar masses)
;       EM - The [-/+] 1-sigma uncertainties on mass
;       TEFF - The most probable Teff (Kelvin)
;       ETEFF - The [-/+] 1-sigma uncertainties on Teff
;       LOGG - The most probable log g (cm/s)
;       ELOGG - The [-/+] 1-sigma uncertainties on log g
;       RADIUS - The most probable radius (Jupiter radius) 
;       ERADIUS - The [-/+] 1-sigma uncertainties on radius
;       LOGL - The most probable log L
;       ELOGL - The [-/+] 1-sigma uncertainties on log L
;       OUT_STRUCT - An IDL structure containing all of these outputs (much simpler to use).
;                    In that particular case, masses are output in Jupiter masses.
;
; NOTE:
;       Please cite the following papers when using this procedure :
;       Gagné et al. 2014, Baraffe et al. 2003, Allard et al. 2013
;
;       This routine requires the "barrafe_interpol_input.sav" IDL data save file. Please obtain this file at
;       http://sites.google.com/site/mbderg/banyanii_additional_material under section 5.6 and update the
;       "idl_datasave_path" variable at the start of "baraffe_interpol.pro" to a valid path on your computer
;       leading to the file.
;
; PROCEDURES USED:
;       DELETEVAR, BARAFFE_INTERPOL, INTEGRAL, INTERPOL2
;
; REFERENCES:
;       Allard, F., Homeier, D., Freytag, B., Schaffenberger, & Rajpurohit, A. S. 2013, MmScAI, 24, 128
;       Baraffe, I., Chabrier, G., Barman, T. S., Allard, F., & Hauschildt, P. H. 2003, A&A, 402, 701
;       Gagné, J., Lafrenière, D., Doyon, R., Malo, L., & Artigau, É. 2014, ApJ, 783, 121
;       
; MODIFICATION HISTORY:
;       WRITTEN, Jonathan Gagne, August, 21 2013
;       MAJOR UPDATE, Jonathan Gagne, June, 23 2015 - Updated BT-Settl models, added outputs, made more robust
;-
Pro banyan_mass, age1, age2, m, em, teff, eteff, logg, elogg, radius, eradius, logl, elogl, $
                   J=J, H=H, K=K, W1=W1, W2=W2, EJ=EJ, EH=EH, EK=EK, EW1=EW1, EW2=EW2, DIST=dist, $
                   EDIST=edist, LI_MARG=li_marg, MAX_MASS=max_mass, BINARY=binary, S2M=s2m, SWISE=swise, $
                   AGES=ages, ALLWISE=allwise, NAGE=nage, SIMBADNAME=simbadname, PDF_MASS=pdf_mass, $
                   NP_MODEL=np_model, PDF_RADIUS=pdf_radius, PDF_TEFF=pdf_teff, PDF_LOGG=pdf_logg, $
                   PDF_LOGL=pdf_logl, AMES_COND=ames_cond, out_struct=out_struct
    
  forward_function deletevar, baraffe_interpol, integral, interpol2
  nan = !values.f_nan
  
  if ~keyword_set(np_model) then $
    np_model = 15000L
  
  if keyword_set(nage) then $
    np = nage else $
    np = 100.
  if ~ keyword_set(max_mass) then $
    max_mass = 0.2
  
  if keyword_set(age1) then begin
    if size(age1,/TYPE) eq 7L then begin
      case age1 of
        'twhydra' : begin & age1 = 5. & age2 = 15. & end
        'betapic' : begin & age1 = 20. & age2 = 26. & end
        'tucana' : begin & age1 = 20. & age2 = 40. & end
        'carina' : begin & age1 = 20. & age2 = 40. & end
        'columba' : begin & age1 = 20. & age2 = 40. & end
        'argus' : begin & age1 = 30. & age2 = 50. & end
        'abdora' : begin & age1 = 110. & age2 = 130. & end
        'carnear' : begin & age1 = 200.*(1.-.15) & age2 = 200.*(1.+.15) & end
        'ursamaj' : begin & age1 = 400. & age2 = 600. & end
        'herlyra' : begin & age1 = 250.*(1.-.15) & age2 = 200.*(1.+.15) & end
      endcase
    endif
  endif
  
  if ~keyword_set(dist) and ~keyword_set(nowarning) then $
    message, ' Did not receive a distance; Please make sure that you have input absolute magnitudes !', /continue
  
  if keyword_set(s2m) then begin
    J = s2m.J_M
    H = s2m.H_M
    K = s2m.K_M
    EJ = s2m.J_CMSIG
    EH = s2m.H_CMSIG
    EK = s2m.K_CMSIG
  endif
  
  if keyword_set(swise) then begin
    if isa(swise,'struct') then begin
      W1 = swise.W1MPRO
      W2 = swise.W2MPRO
      EW1 = swise.W1SIGMPRO
      EW2 = swise.W2SIGMPRO
    endif
  endif
  
  if ~keyword_set(j) and ~keyword_set(h) and ~keyword_set(k) and $
     ~keyword_set(w1) and ~keyword_set(w2) then begin
    em = [nan,nan]
    m = nan
    return
  endif  
  
  if keyword_set(dist) and keyword_set(edist) then begin
    dmod = 5.*alog10(dist)-5.
    edmod = edist/dist*5./alog(10)
  endif
  if J eq !NULL or EJ eq !NULL then begin
    J = nan
    EJ = nan
  endif else if keyword_set(dmod) then begin
    J -= dmod
    EJ = sqrt(EJ^2 + edmod^2)
  endif
  if H eq !NULL or EH eq !NULL then begin
    H = nan
    EH = nan
  endif else if keyword_set(dmod) then begin
    H -= dmod
    EH = sqrt(EH^2 + edmod^2)
  endif
  if K eq !NULL or EK eq !NULL then begin
    K = nan
    EK = nan
  endif else if keyword_set(dmod) then begin
    K -= dmod
    EK = sqrt(EK^2 + edmod^2)
  endif
  if W1 eq !NULL or EW1 eq !NULL then begin
    W1 = nan
    EW1 = nan
  endif else if keyword_set(dmod) then begin
    W1 -= dmod
    EW1 = sqrt(EW1^2 + edmod^2)
  endif
  if W2 eq !NULL or EW2 eq !NULL then begin
    W2 = nan
    EW2 = nan
  endif else if keyword_set(dmod) then begin
    W2 -= dmod
    EW2 = sqrt(EW2^2 + edmod^2)
  endif
  
  if keyword_set(binary) then begin
    J += alog10(2.)*2.5
    H += alog10(2.)*2.5
    K += alog10(2.)*2.5
    W1 += alog10(2.)*2.5
    W2 += alog10(2.)*2.5
  endif
  
  ages = findgen(np)/(float(np)-1.)*(max([age1,age2])-min([age1,age2]))+min([age1,age2])
  nage = n_elements(ages)
  if nage eq 1L then $
    ages = age1[0L]
  
  mass = !NULL
  for i=0L, np-1L do begin & $
    
    deletevar, mass, jt, ht, kt, w1t, w2t
    baraffe_interpol, ages[i], mass, jt, ht, kt, w1t, w2t, Lit, tefft, loggt, radiust, loglt, $
      MAX_MASS=max_mass, NVEC=NP_MODEL, AMES_COND=ames_cond, STRUC=STRUC_BARAFFE, WSTRUC=WSTRUC_BARAFFE & $
    if keyword_set(stop) then stop
    
    if ~keyword_set(pdf_2d) then begin & $
      pdf_2d = dblarr(np,n_elements(mass)) + !values.d_nan & $
      pdf_J = pdf_2d & $
      pdf_H = pdf_2d & $
      pdf_K = pdf_2d & $
      pdf_W1 = pdf_2d & $
      pdf_W2 = pdf_2d & $
    endif
    if ~keyword_set(teff_2d) then $
      teff_2D = dblarr(np,n_elements(mass)) + !values.d_nan
    if ~keyword_set(logg_2d) then $
      logg_2D = dblarr(np,n_elements(mass)) + !values.d_nan
    if ~keyword_set(rad_2d) then $
      rad_2D = dblarr(np,n_elements(mass)) + !values.d_nan
    if ~keyword_set(logl_2d) then $
      logl_2d = dblarr(np,n_elements(mass)) + !values.d_nan
    
    pdf = dblarr(n_elements(jt))+0d0 & $
    OK = 0L
    if finite(J*EJ) then begin & $
      probJ = alog10(1./(sqrt(2.*!pi)*EJ)*exp(-(J-JT)^2/(2d0*EJ^2))) & $
      pdf += probJ & $
      OK = 1 & $
    endif & $
    if finite(H*EH) then begin & $
      probH = alog10(1./(sqrt(2.*!pi)*EH)*exp(-(H-HT)^2/(2d0*EH^2))) & $
      pdf += probH & $
      OK = 1 & $
    endif & $
    if finite(K*EK) then begin & $
      probK = alog10(1./(sqrt(2.*!pi)*EK)*exp(-(K-KT)^2/(2d0*EK^2))) & $
      pdf += probK & $
      OK = 1 & $
    endif & $
    if finite(W1*EW1) then begin & $
      probW1 = alog10(1./(sqrt(2.*!pi)*EW1)*exp(-(W1-W1T)^2/(2d0*EW1^2))) & $
      pdf += probW1 & $
      OK = 1 & $
    endif & $
    if finite(W2*EW2) then begin & $
      probW2 = alog10(1./(sqrt(2.*!pi)*EW2)*exp(-(W2-W2T)^2/(2d0*EW2^2))) & $
      pdf += probW2 & $
      OK = 1 & $
    endif & $
    
    if OK eq 0L then begin & $
      em = [1,1]+!values.f_nan & $
      m = !values.f_nan & $
      return & $
    endif & $
  
    pdf_2d[i,*] = double(pdf)
    if keyword_set(probj) then $
      pdf_J[i,*] = double(probJ)
    if keyword_set(probh) then $
      pdf_H[i,*] = double(probH)
    if keyword_set(probk) then $
      pdf_K[i,*] = double(probK)
    if keyword_set(probw1) then $
      pdf_W1[i,*] = double(probW1)
    if keyword_set(probw2) then $
      pdf_W2[i,*] = double(probW2)
    
    ;Store model values
    TEFF_2D[i,*] = tefft
    LOGG_2D[i,*] = loggt
    RAD_2D[i,*] = radiust
    LOGL_2D[i,*] = loglt
    
  endfor
  
  pdf_2d -= max(pdf_2d,/nan)
  pdf_2d = 10d^pdf_2d
  pdf_J -= max(pdf_J,/nan)
  pdf_J = 10d^pdf_J
  pdf_H -= max(pdf_H,/nan)
  pdf_H = 10d^pdf_H
  pdf_K -= max(pdf_K,/nan)
  pdf_K = 10d^pdf_K
  pdf_W1 -= max(pdf_W1,/nan)
  pdf_W1 = 10d^pdf_W1
  pdf_W2 -= max(pdf_W2,/nan)
  pdf_W2 = 10d^pdf_W2
  
  age_prior = 1./(sqrt(2.*!pi)*abs(age1-age2))*exp(-(ages-mean([age1,age2]))^2/(2d0*abs(age1-age2)^2))
  age_prior2d = age_prior#make_array((size(pdf_2d))[2],value=1.,/float)
  
  PDF_marg = total(pdf_2d*age_prior2d,1,/nan,/double);total(pdf_2d,2,/nan)*age_prior
  xv = mass
  
  cdf = integral(xv, pdf_marg,/accumulate)
  nmes = max(cdf,/nan)
  ;Find where the CDF gets to half its max value. This will be the central value
  void = min(abs(cdf-double(nmes)/2d0),wm)
  xf = xv[wm]
  pdf_mass = [[mass], [pdf_marg]]
  ;What width covers 68% of total area ? This will give the 1-sigma uncertainties
  void = min((abs(abs(cdf-cdf[wm])/double(nmes)*2d0-erf(1d0/sqrt(2d0))))[0L:wm],wm1)
  void = min((abs(abs(cdf-cdf[wm])/double(nmes)*2d0-erf(1d0/sqrt(2d0))))[wm:*],wm2)
  wm2 += wm
  exf = [xv[wm]-xv[wm1],xv[wm2]-xv[wm]]
  m = xf
  em = exf
  
  ;Create a 2D-PDF as a function of age and Teff
  nps = n_elements(mass)
  teff_range = [min(teff_2d,/nan),max(teff_2d,/nan)]
  teff_vec = dindgen(nps)/double(nps-1L)*(teff_range[1]-teff_range[0])+teff_range[0]
  pdf_teff_2d = dblarr(np,nps)+!values.d_nan
  for i=0L, np-1L do $
    pdf_teff_2d[i,*] = interpol2(reform(pdf_2d[i,*]),reform(teff_2d[i,*]),teff_vec,/repair_nan)
  pdf_teff_marg = total(pdf_teff_2d*age_prior2d,1,/nan,/double)
  teff_marg = total(pdf_teff_2d*(make_array(np,value=1.,/float)#pdf_teff_marg)*age_prior2d,/nan,/double,1)/total((make_array(np,value=1.,/float)#pdf_teff_marg)*age_prior2d,/nan,/double,1)
  pdf_teff = [[teff_vec], [teff_marg]]
  
  ;Repeat for Teff
  xv = teff_vec
  cdf_teff = integral(xv, pdf_teff_marg,/accumulate)
  nmes = max(cdf_teff,/nan)
  void = min(abs(cdf_teff-double(nmes)/2d0),wm)
  xf = xv[wm]
  void = min((abs(abs(cdf_teff-cdf_teff[wm])/double(nmes)*2d0-erf(1d0/sqrt(2d0))))[0L:wm],wm1)
  void = min((abs(abs(cdf_teff-cdf_teff[wm])/double(nmes)*2d0-erf(1d0/sqrt(2d0))))[wm:*],wm2)
  wm2 += wm
  exf = [xv[wm]-xv[wm1],xv[wm2]-xv[wm]]
  teff = xf
  eteff = exf
  
  ;Repeat for log g
  logg_range = [min(logg_2d,/nan),max(logg_2d,/nan)]
  logg_vec = dindgen(nps)/double(nps-1L)*(logg_range[1]-logg_range[0])+logg_range[0]
  pdf_logg_2d = dblarr(np,nps)+!values.d_nan
  for i=0L, np-1L do $
    pdf_logg_2d[i,*] = interpol2(reform(pdf_2d[i,*]),reform(logg_2d[i,*]),logg_vec,/repair_nan)
  pdf_logg_marg = total(pdf_logg_2d*age_prior2d,1,/nan,/double)
  logg_marg = total(pdf_logg_2d*(make_array(np,value=1.,/float)#pdf_logg_marg)*age_prior2d,/nan,/double,1)/total((make_array(np,value=1.,/float)#pdf_logg_marg)*age_prior2d,/nan,/double,1)
  pdf_logg = [[logg_vec], [logg_marg]]
  xv = logg_vec
  cdf_logg = integral(xv, pdf_logg_marg,/accumulate)
  nmes = max(cdf_logg,/nan)
  void = min(abs(cdf_logg-double(nmes)/2d0),wm)
  xf = xv[wm]
  void = min((abs(abs(cdf_logg-cdf_logg[wm])/double(nmes)*2d0-erf(1d0/sqrt(2d0))))[0L:wm],wm1)
  void = min((abs(abs(cdf_logg-cdf_logg[wm])/double(nmes)*2d0-erf(1d0/sqrt(2d0))))[wm:*],wm2)
  wm2 += wm
  exf = [xv[wm]-xv[wm1],xv[wm2]-xv[wm]]
  logg = xf
  elogg = exf
  
  ;Repeat for Radius
  rad_range = [min(rad_2d,/nan),max(rad_2d,/nan)]
  rad_vec = dindgen(nps)/double(nps-1L)*(rad_range[1]-rad_range[0])+rad_range[0]
  pdf_rad_2d = dblarr(np,nps)+!values.d_nan
  for i=0L, np-1L do $
    pdf_rad_2d[i,*] = interpol2(reform(pdf_2d[i,*]),reform(rad_2d[i,*]),rad_vec,/repair_nan)
  pdf_rad_marg = total(pdf_rad_2d*age_prior2d,1,/nan,/double)
  rad_marg = total(pdf_rad_2d*(make_array(np,value=1.,/float)#pdf_rad_marg)*age_prior2d,/nan,/double,1)/total((make_array(np,value=1.,/float)#pdf_rad_marg)*age_prior2d,/nan,/double,1)
  pdf_radius = [[rad_vec], [rad_marg]]
  xv = rad_vec
  cdf_rad = integral(xv, pdf_rad_marg,/accumulate)
  nmes = max(cdf_rad,/nan)
  void = min(abs(cdf_rad-double(nmes)/2d0),wm)
  xf = xv[wm]
  void = min((abs(abs(cdf_rad-cdf_rad[wm])/double(nmes)*2d0-erf(1d0/sqrt(2d0))))[0L:wm],wm1)
  void = min((abs(abs(cdf_rad-cdf_rad[wm])/double(nmes)*2d0-erf(1d0/sqrt(2d0))))[wm:*],wm2)
  wm2 += wm
  exf = [xv[wm]-xv[wm1],xv[wm2]-xv[wm]]
  radius = xf
  eradius = exf
  
  ;Repeat for Log L
  logl_range = [min(logl_2d,/nan),max(logl_2d,/nan)]
  logl_vec = dindgen(nps)/double(nps-1L)*(logl_range[1]-logl_range[0])+logl_range[0]
  pdf_logl_2d = dblarr(np,nps)+!values.d_nan
  for i=0L, np-1L do $
    pdf_logl_2d[i,*] = interpol2(reform(pdf_2d[i,*]),reform(logl_2d[i,*]),logl_vec,/repair_nan)
  pdf_logl_marg = total(pdf_logl_2d*age_prior2d,1,/nan,/double)
  logl_marg = total(pdf_logl_2d*(make_array(np,value=1.,/float)#pdf_logl_marg)*age_prior2d,/nan,/double,1)/total((make_array(np,value=1.,/float)#pdf_logl_marg)*age_prior2d,/nan,/double,1)
  pdf_logl = [[logl_vec], [logl_marg]]
  xv = logl_vec
  cdf_logl = integral(xv, pdf_logl_marg,/accumulate)
  nmes = max(cdf_logl,/nan)
  void = min(abs(cdf_logl-double(nmes)/2d0),wm)
  xf = xv[wm]
  void = min((abs(abs(cdf_logl-cdf_logl[wm])/double(nmes)*2d0-erf(1d0/sqrt(2d0))))[0L:wm],wm1)
  void = min((abs(abs(cdf_logl-cdf_logl[wm])/double(nmes)*2d0-erf(1d0/sqrt(2d0))))[wm:*],wm2)
  wm2 += wm
  exf = [xv[wm]-xv[wm1],xv[wm2]-xv[wm]]
  logl = xf
  elogl = exf
  
  ;Create an output structure with masses in M_Jup
  pdf_mass2 = pdf_mass
  pdf_mass2[*,0] *= 1047.6
  out_struct = {mass:m*1047.6,emass:em*1047.6,radius:radius,eradius:eradius,$
    teff:teff,eteff:eteff,logg:logg,elogg:elogg,$
    logl:logl,elogl:elogl,pdf_mass:pdf_mass2,pdf_radius:pdf_radius,$
    pdf_teff:pdf_teff,pdf_logg:pdf_logg,pdf_logl:pdf_logl}
  
End